import pygame.sprite
from idle.utils import Vec2
from idle.data import loadimage
import numpy

class Background(object):
    def __init__(self, image, rect, pixelated):
        self.image = loadimage(image)
        
        self.image_copy = self.image.copy()
        self.pixelated = pixelated
        self.prev_pixelated = 1
        self.rect = rect
        self.image_size = self.image.get_size()

        self.display_image = pygame.Surface((self.rect.width, self.rect.height))
        self.display_image.blit(self.image_copy, self.rect)
            
        self.regen_pixelated()

    def update(self, dt):
        if self.prev_pixelated != self.pixelated and self.pixelated > 1:
            self.regen_pixelated()
            self.prev_pixelated = self.pixelated

    def draw(self, screen):
        if self.pixelated > 1:
            screen.blit(self.display_image, self.rect)
        else:
            screen.blit(self.image_copy, self.rect)

    def regen_pixelated(self):
        if self.pixelated > 1:
            blocks = self.pixelated * 3
            for y in range(0, self.image_size[1], blocks):
                for x in range(0, self.image_size[0], blocks):
                    rtile = pygame.Rect([x, y, blocks, blocks])
                    #print rtile
                    color = pygame.transform.average_color(self.image_copy, rtile)
                    self.display_image.fill(color, rtile)
           