import pygame
from idle.gamesprite import GameSprite
from idle.utils import Vec2


class BgActor(object):
    def __init__(self, bgaconfig, pixelated):
        self.name = bgaconfig['name']
        self.walks = False
        self.cutscene = False
        if bgaconfig['walking'] is not False:
            if bgaconfig['walking'] == 'cutscene':
                self.cutscene = True
                self.walk_steps = []
                self.speed = bgaconfig['walking_speed']
                self.walks = False
            else:
                self.walk_steps = bgaconfig['walking']['steps']
                self.speed = bgaconfig['walking']['speed']  
                self.walks = True

            self.curr_walk_step = 0
            self.curr_timer = 0
            self.curr_timer_limit = 0
            self.step_timer = 0
            self.step = False
            self.direction = 'left'
            self.from_x = []
        self.pixelated = pixelated

        self.dialog = bgaconfig['dialog']

        self.rects = []
        self.sprites = []
        for rect_conf in bgaconfig['instances']:
            r = rect_conf['rect']
            rect = pygame.Rect(r[0], r[1], r[2], r[3])
            self.rects.append(rect)

            gs = GameSprite(bgaconfig['image'], rect_conf['scale'], rect, bgaconfig['animated'], pixelated)
            if 'first_frame' in bgaconfig and bgaconfig['first_frame']:
                gs.curr_frame = bgaconfig['first_frame']
            if self.walks:
                gs.manual = True
            if self.cutscene:
                gs.manual = True
            self.sprites.append(gs)
        



    def update(self, dt):

        if self.walks:
            
            if self.curr_walk_step > len(self.walk_steps) - 1:
                self.curr_walk_step = 0
            
            step = self.walk_steps[self.curr_walk_step].split(' ')

            if step[0] == 'wait':
                self.curr_timer_limit = int(step[1])
                self.step = False
                for s in self.sprites:
                    if self.direction == 'left':
                        s.curr_frame = 3
                    else:
                        s.curr_frame = 2

            if step[0] == 'left' or step[0] == 'right':
                self.curr_timer_limit = int(step[2])

                if self.curr_timer == 0:
                    self.from_x = []
                    for r in self.rects:
                        self.from_x.append(r.x)

            if step[0] == 'left':
                self.direction = 'left'
                for x, r, s in zip(self.from_x, self.rects, self.sprites):
                    s.rect.x = self.move(x, self.curr_timer, -int(step[1]), self.curr_timer_limit)
                    s.curr_frame = 3

            if step[0] == 'right':
                self.direction = 'right'
                for x, r, s in zip(self.from_x, self.rects, self.sprites):
                    s.rect.x = self.move(x, self.curr_timer, int(step[1]), self.curr_timer_limit)
                    s.curr_frame = 2

            self.curr_timer += dt

            if self.curr_timer > self.curr_timer_limit:
                if step[0] == 'left' or step[0] == 'right':
                    self.step = False
                    for r, s in zip(self.rects, self.sprites):
                        r.x = s.rect.x
                        r.y = s.rect.y
                if step[0] == 'left':
                    for s in self.sprites:
                        s.curr_frame = 3
                if step[0] == 'right':
                    for s in self.sprites:
                        s.curr_frame = 2
                if step[0] == 'wait':
                    self.step = False

                self.curr_timer = 0
                self.curr_walk_step += 1

            self.step_timer += dt
            if self.step_timer > self.speed and step[0] != 'wait':
                self.step_timer = 0
                self.step = not self.step


        for s in self.sprites:
            s.pixelated = self.pixelated
            s.update(dt)

    def move(self, start, time, change, duration):
        return (change * (time/float(duration))) + start

    def draw(self, screen):
        for s in self.sprites:
            if self.walks and self.step and self.direction == 'left':
                s.curr_frame += 1
            if self.walks and self.step and self.direction == 'right':
                s.curr_frame -= 1
            s.draw(screen)
            if self.walks and self.step and self.direction == 'left':
                s.curr_frame -= 1
            if self.walks and self.step and self.direction == 'right':
                s.curr_frame += 1
            

    def collidepoint(self, pos):
        for rect in self.rects:
            if rect.collidepoint(pos):
                return True
        return False
