import pygame
from idle.textwriter import TextWriter
from idle.data import *
from idle.constants import *

MAX_CHAR_WIDTH = 32

class Choice(object):
    def __init__(self, dest, pos, rows, size):
        self.dest = dest
        self.rect = pygame.Rect(pos[0], pos[1], 1, 1)
        self.active = False
        self.rows = rows
        self.rect.height = (size+5) * (len(rows)+1)
        self.rect.width = size*MAX_CHAR_WIDTH

class Choices(object):
    def __init__(self, locale, choicelines, pixelated):
        self.locale = locale
        self.choices = []
        self.size = 18
        self.text = pygame.font.Font(filepath(DIALOG_FONT), self.size)
        self.choice_click_handled = True
        self.aliasing = True
        self.pixelated = pixelated

        self.textbuf_dirty = True
        
        prev_y = 250
        for line in choicelines:
            inst = line.split('%')
            dest = inst[0]
            option = inst[1]
            olines = self.load_rows(option)
            c = Choice(dest, (0, prev_y), olines, self.size)
            self.choices.append(c)
            prev_y += c.rect.height


    def load_rows(self, line):
        words = line.split(' ')
        rows = [""]
        curr_row = 0
        for w in words:
            if len(rows[curr_row]) + len(w) < MAX_CHAR_WIDTH:
                rows[curr_row] += w
                rows[curr_row] += " "
            else:
                curr_row += 1
                rows.append(w + " ")
        return rows

    def update(self, screen):
        mousestate = pygame.mouse.get_pressed()
        mouse_pos = pygame.mouse.get_pos()
        for c in self.choices:
            if c.rect.collidepoint(mouse_pos):
                c.active = True
                self.textbuf_dirty = True
                if mousestate[0] and not self.choice_click_handled:
                    self.locale.dialog_switching = True
                    self.locale.dialog_dest = c.dest

                self.choice_click_handled = mousestate[0]
            else:
                if c.active == True:
                    self.textbuf_dirty = True
                c.active = False

    def draw(self, screen):
        hilite = [255,155,255]
        color = [235,235,255]

        if self.textbuf_dirty:
            self.texbuf = pygame.Surface((420, RESOLUTION_Y)).convert_alpha()
            self.texbuf.fill([0,0,0,0])
            for c in self.choices:
                for i, row in enumerate(c.rows):
                    col = color
                    if c.active:
                        col = hilite
                    t = self.text.render(row, self.aliasing, col)
                    self.texbuf.blit(t, (40, c.rect.y + (i * self.size)))

            blocks = self.pixelated - 4
            if blocks > 2:
                size = self.texbuf.get_size()
                for y in range(0, size[1], self.pixelated):
                    for x in range(0, size[0], self.pixelated):
                        rtile = pygame.Rect([x, y, self.pixelated, self.pixelated])
                        color = pygame.transform.average_color(self.texbuf, rtile)
                        self.texbuf.fill(color, rtile)
            self.textbuf_dirty = False
        screen.blit(self.texbuf, (40,0))
