from idle.bgactor import BgActor
from idle.locale import Locale
from idle.dialog import Dialog

class Cutscene(object):
    def __init__(self, game, cutscene):

        self.steps = cutscene['steps']
        self.curr_step = 0
        self.curr_timer = 0
        self.curr_timer_max = 0
        self.game = game
        self.locale = Locale(cutscene['locale'][0], cutscene['locale'][1], game)
        self.locale.cutscene = True

        self.extras = []
        self.extras_conf = cutscene['extras']
        for extra_conf in self.extras_conf:
            e = BgActor(extra_conf, game.pixelated)
            self.extras.append(e)

        self.active_dialog = False
        self.finished = False

    def get_next_step(self):
        if len(self.steps) - 1 > self.curr_step:
            self.curr_step += 1
            self.curr_timer = 0
        else:
            self.finished = True
            self.locale.cutscene = False
            self.game.active_class = self.locale

    def update(self, dt):
        self.locale.update(dt)

        for e in self.extras:
            if self.steps[self.curr_step]['type'] == 'extramove':
                inst = self.steps[self.curr_step]['value'].split(' ')
                moving = inst[0]
                step = inst[1:]
                self.curr_timer_max = int(inst[-1:][0])
                if e.name == moving:
                    if len(e.walk_steps) < 1:
                        walkinst = ' '.join(step)
                        e.walk_steps.append(walkinst)
                        e.walks = True
                    e.update(dt)
                    self.curr_timer += dt
            else:
                e.walk_steps = []
                e.walks = False

        if self.steps[self.curr_step]['type'] == 'dialog':
            if self.active_dialog == False:
                self.active_dialog = Dialog(self.locale, self.steps[self.curr_step]['value'])
            if self.active_dialog.finished:
                self.active_dialog = False
                self.get_next_step()

        if self.steps[self.curr_step]['type'] == 'wait':
            self.curr_timer_max = self.steps[self.curr_step]['value']
            self.curr_timer += dt
            if self.curr_timer > self.curr_timer_max:
                self.get_next_step()

        if self.active_dialog != False:
            self.active_dialog.update(dt)

        #print self.curr_step, self.curr_timer, self.curr_timer_max, self.steps[self.curr_step]

        if self.curr_timer > self.curr_timer_max:
            if self.steps[self.curr_step] == "extramove":
                e.walk_steps = []
                e.walks = False
            self.get_next_step()


    def draw(self, screen):
        self.locale.draw(screen)

        for e in self.extras:
            e.draw(screen)

        if self.locale.blockers > 0:
            for blocker in self.locale.blockers:
                screen.blit(blocker['image'], blocker['rect'])

        if self.active_dialog != False:
            self.active_dialog.draw(screen)
