import pygame
import math
from idle.constants import *
from idle.data import * 
from idle.textwriter import TextWriter
from idle.gamesprite import GameSprite
from idle.choices import Choices
from idle.background import Background

ACTOR_WIDTH = 540

class Dialog(object):
    def __init__(self, locale, script):
        self.curr_pose = 0
        self.prev_pose = self.curr_pose
        self.curr_actor = 0
        self.next_actor = 0

        self.locale = locale
        self.names = []
        self.pretty_names = []
        self.actors = []
        self.script = self.parse_script(filepath(script))
        
        self.text = TextWriter()

        self.dialog_rect = pygame.Rect(20, 20, RESOLUTION_X * .4, RESOLUTION_Y - 40)
        self.text_width = 32 # characters that can fit on a row

        self.choice = False
        self.text.writing_text = True
        self.get_next_script()

        self.actor_slide_timer = 0
        self.slide_actor_in = True
        self.slide_actor_out = False
        self.switching_actors = False

        self.actor_offset = ACTOR_WIDTH

        self.finished = False

        self.scroll_wait = False

        self.pixelated = 0
        self.clickhandled = True


    def parse_script(self, script_path):
        f = file(script_path)
        output = []
        for i, line in enumerate(f):
            if i is 0:
                actors = line.lstrip('[').rstrip(']\n').rstrip(']\r').split(',')
                for a in actors:
                    if len(a) < 1:
                        continue
                    parts = a.split(':')
                    name = parts[0]
                    self.names.append(name)
                    path = parts[1]
                    actor = self.locale.game.actors[path]
                    poses = []
                    self.pretty_names.append(actor['name'])
                    if len(actor['poses']) > 0:
                        for p in actor['poses']:
                            poses.append(GameSprite(
                                p, 
                                1,
                                pygame.Rect(RESOLUTION_X, 0, ACTOR_WIDTH, RESOLUTION_Y), 
                                False, 
                                0
                            ))
                    else:
                        self.slide_actor_in = False
                    self.actors.append(poses)
            else:
                output.append(line)
        return output

    def get_next_script(self):
        if self.script != False and len(self.script) > 0:
            line = self.script.pop(0).strip()
            if line[0] is '!':
                inst = line[1:].split(' ')
                if inst[0] == 'pose':
                    self.curr_pose = int(inst[1]) - 1

                if inst[0] == 'color':
                    self.text.set_color(inst[1])

                if inst[0] == 'actor':
                    self.next_actor = int(inst[1]) - 1
                    self.switching_actors = True
                    self.actor_slide_timer = 0
                    self.slide_actor_in = False
                    self.slide_actor_out = True

                if inst[0] == 'choice':
                    choicelines = [self.script.pop(0).strip() for _ in range(0, int(inst[1]))]
                    self.choice = Choices(self.locale, choicelines, self.pixelated)
                    self.text.scroll_wait = True
                if inst[0] == 'dialog':
                    if len(inst) > 2:
                        bgactor = inst[1]
                        new_dialog = inst[2]
                    else:
                        bgactor = self.names[self.curr_actor]
                        new_dialog = inst[1]
                    self.locale.set_dialog(bgactor, new_dialog)

                if inst[0] == 'rez_minus_1':
                    self.locale.game.pixelated += 1

                if inst[0] == 'rez_plus_1':
                    if self.locale.game.pixelated > 0:
                        self.locale.game.pixelated -= 1

                if inst[0] == 'rez_heal':
                    self.locale.game.pixelated = 0

                if inst[0] == 'set':
                    self.locale.game.state[inst[1]] = True

                if inst[0] == 'check':
                    val = self.locale.game.state[inst[1]] 
                    if not val:
                        self.script = [" ".join(inst[2:])]

                if inst[0] == 'bg':
                    bg = Background(inst[1], pygame.Rect(0, 0, RESOLUTION_X, RESOLUTION_Y), 0)
                    del(self.locale.bgimage)
                    self.locale.bgimage = bg
                    self.locale.game.locales[self.locale.name]['background'] = inst[1]

                if inst[0] == 'moveactor':
                    self.locale.moveactor(self.names[self.curr_actor], inst[1])

                if self.choice == False:
                    self.get_next_script()

            else:
                self.text.curr_rows = self.load_rows(line)
                self.text.curr_line = line
                self.text.curr_charcount = len(line)

                self.text.writing_text = True
                self.text.scroll_wait = False
        else:
            self.text.writing_text = False
            self.text.scroll_wait = False
            self.finished = True

        if self.choice != False and self.text.scroll_wait:
            self.text.curr_char = self.text.curr_charcount

    def load_rows(self, line):
        words = line.split(' ')
        rows = [""]
        curr_row = 0
        for w in words:
            if len(rows[curr_row]) + len(w) < 32:
                rows[curr_row] += w
                rows[curr_row] += " "
            else:
                curr_row += 1
                rows.append(w + " ")
        return rows

    def update(self, dt):

        #if self.writing_text:
        self.text.update(dt)

        mousestate = pygame.mouse.get_pressed()
        if mousestate[0] and not self.clickhandled:

            pos = pygame.mouse.get_pos()
            if pos[0] < RESOLUTION_X * .4:
                if len(self.script) > 1:
                    choicenext = self.script[0].startswith('!choice')
                else:
                    choicenext = False
                if self.text.scroll_wait and not choicenext:
                    self.get_next_script()
                    self.text.scroll_wait = False
                    self.text.writing_text = True
                    self.text.curr_char = 0
                else:
                    self.text.curr_char = self.text.curr_charcount
                    self.text.scroll_wait = True

                if choicenext:
                    self.get_next_script()
        self.clickhandled = mousestate[0]

        if len(self.pretty_names) > 0:
            self.text.name = self.pretty_names[self.curr_actor]

        if self.slide_actor_in:# and self.actor_offset > RESOLUTION_X - ACTOR_WIDTH:
            self.actor_slide_timer += dt
            self.actor_offset = self.ease_in(self.actor_slide_timer, RESOLUTION_X, RESOLUTION_X - ACTOR_WIDTH)

        if self.slide_actor_out:
            self.actor_slide_timer += dt
            self.actor_offset = self.ease_out(self.actor_slide_timer, RESOLUTION_X - ACTOR_WIDTH, ACTOR_WIDTH)

        if self.switching_actors and self.actor_offset > RESOLUTION_X:
            self.curr_actor = self.next_actor
            self.switching_actors = False
            self.slide_actor_out = False
            self.slide_actor_in = True
        if len(self.actors) > 0:
            self.actors[self.curr_actor][self.curr_pose].rect.x = self.actor_offset
            self.actors[self.curr_actor][self.curr_pose].pixelated = self.pixelated
            self.actors[self.curr_actor][self.curr_pose].update(dt)

        self.text.pixelated = self.pixelated

        if self.choice != False:
            self.choice.pixelated = self.pixelated
            self.choice.update(dt)
        

    def ease_in(self, time, start, change):
        return start - change * (-math.pow(2.0, int(-10 * time/4)) + 1)

    def ease_out(self, time, start, change):
        return start + change * (math.pow(2.0, 10 * int(time/4 - 1)) + 1)

    def draw(self, screen):
        if len(self.actors) > 0:
            self.actors[self.curr_actor][self.curr_pose].draw(screen)
            
        if self.text.writing_text or self.text.scroll_wait:
            surf = pygame.Surface((self.dialog_rect.width, self.dialog_rect.height)).convert_alpha()
            surf.fill([0,0,0,200])
            screen.blit(surf, self.dialog_rect)
           # screen.fill([0,0,0,50], self.dialog_rect)
            self.text.draw(screen)
            if self.choice != False:
                self.choice.draw(screen)

        
        
        #screen.blit(self.poses[self.curr_pose-1], (self.actor_offset, 0))


