import pygame
from pygame.locals  import *
from idle.locale import Locale
from idle.overworld import Overworld
from idle.utils import Vec2
from idle.data import *
from idle.constants import *
from idle.cutscene import Cutscene

class Game(object):

    def __init__(self):
        self.actors = {
            'actors/ccontemplative.ini': {
                'name': 'Captain Contemplative',
                'poses': [
                    'poses/captaincontemplative-pose1.png',
                    'poses/captaincontemplative-pose2.png',
                    'poses/captaincontemplative-pose3.png'
                ]
            },
            'actors/creep.ini': {

                'name': 'Lighthouse Creep',
                'poses': [
                    'poses/lghthscreep_pose1.png',
                    'poses/lghthscreep_pose2.png',
                    'poses/lghthscreep_pose3.png'
                ]
            },
            'actors/foodstandguy.ini': {
                'name': 'Food Stand Guy',
                'poses': [
                    'poses/foodstandguy_pose1.png',
                    'poses/foodstandguy_pose2.png',
                    'poses/foodstandguy_pose3.png'
                ]
            },
            'actors/gurl.ini': {
                'name': 'Gurl',
                'poses': [
                    'poses/Gurl-pose1.png',
                    'poses/Gurl-pose2.png',
                    'poses/Gurl-pose3.png'
                ]
            },
            'actors/paylee.ini': {
                'name': 'Ms. Paylee',
                'poses': [
                    'poses/mspaylee-pose1.png',
                    'poses/mspaylee-pose2.png',
                    'poses/mspaylee-pose3.png'
                ]
            },
            'actors/pharmacist.ini': {
                'name': 'Pharmacist',
                'poses': [
                    'poses/pharmacist_pose1.png',
                    'poses/pharmacist_pose2.png',
                    'poses/pharmacist_pose3.png'
                ]
            },
            'actors/vpriestess.ini': {
                'name': 'Volcano Priestess',
                'poses': [
                    'poses/volcanopriestess_pose1.png',
                    'poses/volcanopriestess_pose2.png',
                    'poses/volcanopriestess_pose3.png'
                ]
            },
            'actors/bonluvr.ini': {
                'name': 'Bon Luvr',
                'poses': [
                    'poses/bonluvr_pose1.png',
                    'poses/bonluvr_pose2.png',
                    'poses/bonluvr_pose3.png'
                ]
            }
        }
        self.locales = {
            'bay': {
                'background': 'locales/bay/IDLE-DubietyBay-Background-BeforeCompletion.png',
                'music': 'music/dubietybay_theme.wav',
                'bgactors': [
                    {
                        'name': 'creep',
                        'dialog': 'dialog/creep/creep_meet.txt',
                        'walking': False,
                        'instances': [
                            {
                                'rect': (950, 462, 140, 168),
                                'scale': .8
                            }
                        ],
                        'image': 'lghthscreep_spritesheet.png',
                        'animated': {
                            'frames': 2,
                            'fps': 4
                        }
                    }
                ]
            },
            'beach': {
                'background': 'locales/beach/IDLE-BlaseBeach-Background.png',
                'music': 'music/blase_beach_theme.wav',
                'bgactors': [
                    {
                        'name':'tuber',
                        'dialog': 'dialog/tuber.txt',
                        'walking': False,
                        'instances': [
                            {
                                'rect': (540, 477, 91, 168),
                                'scale': .9
                            }
                        ],
                        'image': 'tuber_spritesheet.png',
                        'animated': {
                            'frames': 4,
                            'fps': 4
                        }
                    }
                ]
            },
            'farm': {
                'background': 'locales/farm/IDLE-AnxietyFarms-Background.png',
                'music': 'music/anxiety_farm_theme.wav',
                'bgactors': [
                    {
                        'name':'grass',
                        'dialog': False,
                        'walking': False,
                        'instances': [
                            {
                                'rect': (550, 350, 60, 60),
                                'scale': .6
                            },
                            {
                                'rect': (71, 260, 60, 60),
                                'scale': .4
                            },
                            {
                                'rect': (638, 565, 60, 60),
                                'scale': 1
                            }
                        ],
                        'image': 'grass_animation.png',
                        'animated': {
                            'frames': 2,
                            'fps': 8
                        }
                    },
                    {
                        'name':'grass2',
                        'dialog': False,
                        'walking': False,
                        'instances': [
                            {
                                'rect': (71, 350, 60, 60),
                                'scale': .6
                            },
                            {
                                'rect': (550, 260, 60, 60),
                                'scale': .4
                            }
                        ],
                        'image': 'grass_animation.png',
                        'animated': {
                            'frames': 2,
                            'fps': 5
                        }
                    },
                    {
                        'name': 'pharmacist',
                        'dialog': 'dialog/pharmacist/pharmacist_meet.txt',
                        'walking': {
                            'steps':[
                                'wait 10',
                                'left 100 30',
                                'wait 10',
                                'left 10 30',
                                'wait 20',
                                'right 110 30'
                            ],
                            'speed': 6
                        },
                        'instances': [
                            {
                                'rect': (509, 432, 91, 168),
                                'scale': .8
                            }
                        ],
                        'image': 'pharmacist_spritesheet.png',
                        'animated': {
                            'frames': 4,
                            'fps': 4
                        }
                    }
                ]
            },
            'point': {
                'background': 'locales/point/whatsthepoint.png',
                'music': 'music/whatsthepoint_theme.wav',
                'bgactors': [
                    
                ]
            },
            'school': {
                'background': 'locales/school/schooldude.png',
                'music': 'music/melncollie_theme.wav',
                'bgactors': [
                    {
                        'name': 'paylee',
                        'dialog': 'dialog/paylee/paylee_meet.txt',
                        'first_frame': 3,
                        'walking': {
                            'steps':[
                                'wait 10',
                                'left 100 30',
                                'wait 10',
                                'left 10 30',
                                'wait 20',
                                'right 110 30'
                            ],
                            'speed': 6
                        },
                        'instances': [
                            {
                                'rect': (514, 460, 91, 168),
                                'scale': .85
                            }
                        ],
                        'image': 'paylee_spritesheet.png',
                        'animated': {
                            'frames': 4,
                            'fps': 4
                        }
                    }
                ]
            },
            'settlement': {
                'background': 'locales/settlement/the_settlement.png',
                'music': 'music/settlement_theme.wav',
                'blockers': [
                    {
                        'name': 'cart',
                        'image': 'locales/settlement/food_stand.png',
                        'rect': (896, 305, 241, 366)
                    }
                ],
                'bgactors': [
                    {
                        'name':'grass',
                        'dialog': False,
                        'walking': False,
                        'instances': [
                            {
                                'rect': (240, 366, 60, 60),
                                'scale': 1
                            },
                            {
                                'rect': (754, 244, 60, 60),
                                'scale': .5
                            },
                            {
                                'rect': (833, 367, 60, 60),
                                'scale': 1
                            }
                        ],
                        'image': 'grass_animation.png',
                        'animated': {
                            'frames': 2,
                            'fps': 8
                        }
                    },
                    {
                        'name':'cop',
                        'dialog': 'dialog/settlement_cop.txt',
                        'walking': False,
                        'instances': [
                            {
                                'rect': (551, 165, 48, 88),
                                'scale': .6
                            }
                        ],
                        'image': 'settlement_copman.png',
                        'animated': {
                            'frames': 2,
                            'fps': 5
                        }
                    },
                    {
                        'name': 'girl',
                        'dialog': 'dialog/settlement_gurl.txt',
                        'walking': False,
                        'instances': [
                            {
                                'rect': (451, 165, 48, 88),
                                'scale': .6
                            }
                        ],
                        'image': 'settlement_gurl.png',
                        'animated': {
                            'frames': 2,
                            'fps': 7
                        }
                    },
                    {
                        'name': 'foodstandguy',
                        'dialog': 'dialog/foodstandguy/first_question.txt',
                        'first_frame': 3,
                        'walking': {
                            'steps':[
                                'wait 10',
                                'left 100 30',
                                'wait 10',
                                'left 10 30',
                                'wait 20',
                                'right 110 30'
                            ],
                            'speed': 6
                        },
                        'instances': [
                            {
                                'rect': (790, 342, 91, 168),
                                'scale': .85
                            }
                        ],
                        'image': 'foodstandman_spritesheet.png',
                        'animated': {
                            'frames': 4,
                            'fps': 4
                        }
                    }
                ]
            },
            'volcano': {
                'background': 'locales/volcano/bsvolcano_background.png',
                'music': 'music/bsvolcano_theme.wav',
                'bgactors': [
                    {
                        'name': 'vpriestess',
                        'dialog': 'dialog/vpriestess/vpriestess_meet.txt',
                        'walking': False,
                        'instances': [
                            {
                                'rect': (558, 361, 91, 168),
                                'scale': .8
                            }
                        ],
                        'image': 'vpriestess_spritesheet.png',
                        'animated': {
                            'frames': 2,
                            'fps': 6
                        }
                    }
                ]
            },
            'woods': {
                'background': 'locales/woods/woods_man.png',
                'music': 'music/woods_theme.wav',
                'bgactors': [
                    {
                        'name':'grass',
                        'dialog': False,
                        'walking': False,
                        'instances': [
                            {
                                'rect': (585, 346, 60, 60),
                                'scale': .6
                            },
                            {
                                'rect': (680, 400, 60, 60),
                                'scale': .8
                            },
                            {
                                'rect': (798, 348, 60, 60),
                                'scale': .7
                            }
                        ],
                        'image': 'grass_animation.png',
                        'animated': {
                            'frames': 2,
                            'fps': 8
                        }
                    },
                    {
                        'name': 'bonluvr',
                        'dialog': 'dialog/bonluvr/bonluvr_meet.txt',
                        'walking': False,
                        'instances': [
                            {
                                'rect': (682, 190, 91, 168),
                                'scale': .8
                            }
                        ],
                        'image': 'bonluvr_spritesheet.png',
                        'animated': {
                            'frames': 2,
                            'fps': 15
                        }
                    }
                ]
            }
        }

        self.cutscenes = {
            'opening': {
                'locale': ('settlement', self.locales['settlement']),
                'steps': [
                    {
                        "type": "extramove",
                        "value": "Gurl right 290 50"
                    },
                    {
                        "type": "extramove",
                        "value": "Gurl wait 5"
                    },
                    {
                        "type": "dialog",
                        "value": "dialog/opening_dialog.txt"
                    },
                    {
                        "type": "extramove",
                        "value": "Gurl right 500 55"
                    }
                ],
                'extras':[
                    {
                        'name': 'Gurl',
                        'walking': 'cutscene',
                        'walking_speed': 6,
                        'dialog': False,
                        'instances': [{
                                'rect': (390, 342, 91, 168),
                                'scale': .85
                        }],
                        'image': 'villangurl_spritesheet.png',
                        'animated': {
                            'frames': 4,
                            'fps': 4
                        }
                    }
                ]
            },
            'ending': {
                'locale': ('school', self.locales['school']),
                'steps': [
                    {
                        "type": "extramove",
                        "value": "Gurl right 380 30"
                    },
                    {
                        "type": "extramove",
                        "value": "Gurl wait 5"
                    },
                    {
                        "type": "dialog",
                        "value": "dialog/closing_dialog.txt"
                    },
                    {
                        "type": "extramove",
                        "value": "Gurl right 500 55"
                    }
                ],
                'extras':[
                    {
                        'name': 'Gurl',
                        'walking': 'cutscene',
                        'walking_speed': 6,
                        'dialog': False,
                        'instances': [{
                                'rect': (-19, 455, 91, 168),
                                'scale': .9
                        }],
                        'image': 'villangurl_spritesheet.png',
                        'animated': {
                            'frames': 4,
                            'fps': 4
                        }
                    }
                ]
            }
        }

        self.captain = {
                        'name': 'ccontemplative',
                        'dialog': 'dialog/ccontemplative/ccontemplative_meet.txt',
                        'walking': {
                            'steps':[
                                'wait 10',
                                'left 100 30',
                                'wait 10',
                                'left 10 30',
                                'wait 20',
                                'right 110 30'
                            ],
                            'speed': 6
                        },
                        'instances': [
                            {
                                'rect': (790, 342, 91, 168),
                                'scale': .85
                            }
                        ],
                        'image': 'captaincontemplative-spritesheet.png',
                        'animated': {
                            'frames': 4,
                            'fps': 4
                        }
                    }
        self.mode = 'locale'
        self.mode_changing = False

        self.is_fading = False
        self.fade_step = 0
        self.fade_direction = 'out'
        self.moving_to = False

        self.mode_class = False

        self.pixelated = 0

        self.quitting = False
        self.exit = False
        self.quitconfirm = False

        self.title_fade_out = True
        self.title = loadimage('idle_titlescreen.png')
        self.title_fade_step = 2

        #self.mode_class = Overworld(self)
        self.mode_class = Cutscene(self, self.cutscenes['opening'])

        self.state = {
            'metpharmacist': False,
            'farm_complete': False,
            'loadofshit': False,
            'metcreep': False,
            'perfectwood': False,
            'lighthouse': False,
            'captainadded': False,
            'captainmoved': False,
            'treasure': False,
            'ending': False,
            'endingstarted': False,
        }

    def update(self, dt):
        if self.is_fading:
            if self.fade_direction is 'out' and self.fade_step < 1:
                self.fade_step += dt * .1
            if self.fade_direction is 'in' and self.fade_step > 0:
                self.fade_step -= dt * .1

            if self.fade_step >= 1:
                if self.moving_to:
                    if self.moving_to is 'overworld':
                        self.mode_class = Overworld(self)
                    else:
                        self.mode_class = Locale(self.moving_to, self.locales[self.moving_to], self)
                    self.fade_direction = 'in'

            if self.fade_step <= 0:
                self.is_fading = False

        if self.title_fade_out and self.title_fade_step > 0 :
            self.title_fade_step -= dt * .05

            if self.title_fade_step < 0:
                self.title_fade_out = False

        keystate = pygame.key.get_pressed()

        if keystate[K_y]:
            self.is_fading = True
            self.fade_direction = 'out'

        if keystate[K_h]:
            self.is_fading = True
            self.fade_direction = 'in'

        if keystate[K_i] and self.pixelated < 100:
            self.pixelated += 1
       
        if keystate[K_k] and self.pixelated > 1:
            self.pixelated -= 1

        if keystate[K_ESCAPE]:
            self.quitting = True

        if self.quitting == False:
            self.quitconfirm = False

        if self.quitting and not isinstance(self.mode_class, Credits):
            self.quitconfirm = QuitConfirm(self)

        if self.quitting and isinstance(self.mode_class, Credits):
            self.exit = True

        if self.quitconfirm != False:
            self.quitconfirm.update(dt)
        if self.quitconfirm == False:
            self.mode_class.pixelated = self.pixelated
            self.mode_class.update(dt)

        if self.state['farm_complete'] and self.state['lighthouse'] and not self.state['captainadded']:
            self.locales['point']['bgactors'].append(self.captain)
            self.state['captainadded'] = True

        if self.state['ending'] and not self.state['endingstarted']:
            self.state['endingstarted'] = True
            self.mode_class = Cutscene(self, self.cutscenes['ending'])

        if self.state['endingstarted'] and not isinstance(self.mode_class, Credits) and self.mode_class.finished == True :
            self.mode_class = Credits(self)

    def draw(self, screen):
        self.mode_class.draw(screen)
        if self.quitconfirm != False:
            self.quitconfirm.draw(screen)
        if self.is_fading:
            self.draw_fade(screen)
        if self.title_fade_out:
            self.draw_fade_title(screen)

        if isinstance(self.mode_class, Credits):
            self.mode_class.draw(screen)

    def draw_fade(self, screen):
        if self.fade_step < 0:
            self.fade_step = 0
        if self.fade_step > 1:
            self.fade_step = 1
        alpha = self.fade_step * 255
        surf = pygame.Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()
        surf.fill([0,0,0,alpha])
        screen.blit(surf, (0,0))

    def draw_fade_title(self, screen):
        step = self.title_fade_step
        if self.title_fade_step < 0:
            step = 0
        if self.title_fade_step > 1:
            step = 1
        alpha = step * 255
        surf = pygame.Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()
        surf.blit(self.title, (0,0))
        surf.fill((255,255,255, alpha,), None, pygame.BLEND_RGBA_MULT)
        screen.blit(surf, (0,0))

    def set_dialog(self, locale, bgactor, newdialog):
        for b in self.locales[locale]['bgactors']:
            if b['name'] == bgactor:
                b['dialog'] = newdialog

    def move_bgactor(self, locale, bgactor, target):
        moved = False
        for i,b in enumerate(self.locales[locale]['bgactors']):
            if b['name'] == bgactor:
                b['instances'] = [
                            {
                                'rect': (996, 505, 91, 168),
                                'scale': .85
                            }
                ]
                self.locales[target]['bgactors'].append(b)
                moved = i
        if moved is not False:
            self.locales[locale]['bgactors'] = []

class Credits(object):
    def __init__(self,game):
        self.image = loadimage('idle-creditscreen.png')
        self.locale = game.mode_class.locale
        self.locale.cutscene = True
        self.opacity = 0
        self.timer = 0
        pygame.mixer.music.load(filepath('music/main_theme.wav'))
        pygame.mixer.music.play(-1)


    def update(self, dt):
        if self.opacity < 1:
            self.opacity += dt * .3
        if self.opacity > 1:
            self.opacity = 1
        self.timer += dt
        self.locale.update(dt)

    def draw(self, screen):
        #self.locale.draw(screen)
        buf = pygame.Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()
        buf.blit(self.image, (0,0))
        buf.fill([255,255,255,self.opacity*255], None, pygame.BLEND_RGBA_MULT)
        screen.blit(buf, (0,0))

class QuitConfirm(object):
    def __init__(self, game):
        self.buf = pygame.Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()
        self.buf.fill([0,0,0,175])
        self.game = game
        self.font_size = 24
        self.text = pygame.font.Font(filepath(DIALOG_FONT), self.font_size)

        t = self.text.render('Really Quit?', True, [255,255,255])

        self.buf.blit(t, (455, 200))
        t = self.text.render('Your progress will not be saved.', True, [255,255,255])
        self.buf.blit(t, (350, 229))

        self.yes_color = [255,255,255]
        self.no_color = [255,255,255]

        self.yes_rect = pygame.Rect(340, 260, 150, 150)
        self.no_rect =  pygame.Rect(600, 260, 150, 150)
        self.yes_click_handled = False
        self.no_click_handled = False

    def update(self, dt):
        mousestate = pygame.mouse.get_pressed()
        mouse_pos = pygame.mouse.get_pos()
        if self.yes_rect.collidepoint(mouse_pos):
            self.yes_color = [255,255,155]
            if mousestate[0] and not self.yes_click_handled:
                self.game.exit = True
                pygame.event.clear()
            self.yes_click_handled = mousestate[0]
        else:
            self.yes_color = [255,255,255]
        if self.no_rect.collidepoint(mouse_pos):
            self.no_color = [255,255,155]
            if mousestate[0] and not self.no_click_handled:
                self.game.quitting = False
                pygame.event.clear()
            self.no_click_handled = mousestate[0]

    def draw(self, screen):
        yes = self.text.render('Yes', True, self.yes_color)
        self.buf.blit(yes, (self.yes_rect.x + 50, self.yes_rect.y + 50))
        no = self.text.render('No', True, self.no_color)
        self.buf.blit(no, (self.no_rect.x + 50, self.no_rect.y + 50))
        screen.blit(self.buf, (0,0))
