import pygame.sprite
from idle.utils import Vec2
from idle.data import loadimage

class GameSprite(pygame.sprite.Sprite):
    def __init__(self, image, scale, rect, animated, pixelated):
        
        if scale is not 1:
            rect.width *= scale
            rect.height *= scale
            if animated != False:
                self.image = pygame.transform.smoothscale(loadimage(image), (rect.width * animated['frames'], rect.height))
            else:
                self.image = pygame.transform.smoothscale(loadimage(image), (rect.width, rect.height))
        else:
            self.image = loadimage(image)

        self.image.convert_alpha()
        self.image_copy = self.image.copy()

        self.pixelated = pixelated
        self.prev_pixelated = 1
        self.rect = rect
        self.image_size = self.image.get_size()

        if animated != False:
            self.animated = True
            self.frames = animated['frames']
            self.fps = animated['fps']
            self.curr_frame = 1
            self.time_since_switch = 0
            self.display_image = pygame.Surface((self.rect.width * self.frames, self.rect.height)).convert_alpha()
        else:
            self.animated = False
            self.display_image = pygame.Surface((self.rect.width, self.rect.height)).convert_alpha()
        
        self.manual = False
        self.regen_pixelated()

    def update(self, dt):
        if self.prev_pixelated != self.pixelated and self.pixelated > 1:
            self.regen_pixelated()
            self.prev_pixelated = self.pixelated
        if self.animated and not self.manual:
            self.time_since_switch += dt
            if self.time_since_switch > self.fps:
                if self.curr_frame < self.frames:
                    self.curr_frame += 1
                else:
                    self.curr_frame = 1
                self.time_since_switch = 0

    def draw(self, screen):
        if self.pixelated > 1:
            if self.animated:
                animoffset = pygame.Rect([self.rect.width * (self.curr_frame-1), 0, self.rect.width, self.rect.height])
                screen.blit(self.display_image, self.rect, animoffset)
            else:
                screen.blit(self.display_image, self.rect)
        else:
            if self.animated:
                animoffset = pygame.Rect([self.rect.width * (self.curr_frame-1), 0, self.rect.width, self.rect.height])
                screen.blit(self.image_copy, self.rect, animoffset)
            else:
                screen.blit(self.image_copy, self.rect)

    def regen_pixelated(self):
        self.display_image.fill([0,0,0,0])
        if self.pixelated > 1:
            blocks = self.pixelated * 2
            for y in range(0, self.image_size[1], blocks):
                for x in range(0, self.image_size[0], blocks):
                    rtile = pygame.Rect([x, y, blocks, blocks])
                    color = pygame.transform.average_color(self.image_copy, rtile)
                    self.display_image.fill(color, rtile)
           