import pygame
from pygame.locals import *
from idle.player import Player
from idle.bgactor import BgActor
from idle.keyconfig import KeyConfig
from idle.utils import Vec2
from idle.constants import *
from idle.data import *
from idle.dialog import Dialog
from idle.gamesprite import GameSprite
from idle.background import Background

class Locale(object):
    
    def __init__(self, name, localeconf, game):
        self.bgimage = Background(localeconf['background'], pygame.Rect(0, 0, RESOLUTION_X, RESOLUTION_Y), 0)
        self.name = name
        self.bgactors = []
        self.game = game
        self.pixelated = game.pixelated
        for bga in localeconf['bgactors']:
            self.bgactors.append(BgActor(bga, self.pixelated))

        self.blockers = []
        if 'blockers' in localeconf:
            for blocker in localeconf['blockers']:
                self.blockers.append({
                    'image': loadimage(blocker['image']),
                    'rect': pygame.Rect(blocker['rect']),
                })

        pygame.mixer.music.load(filepath(localeconf['music']))
        pygame.mixer.music.play(-1)

        self.active_dialog = False

        self.font_size = 15
        self.text = pygame.font.Font(filepath(DIALOG_FONT), self.font_size)
        self.exit_click_handled = False
        self.exit_text = "Leave"
        self.exit_rect = pygame.Rect(10, RESOLUTION_Y - 45, self.text.size(self.exit_text)[0] + 20, self.font_size + 20)
        self.exit_buf = pygame.Surface((self.exit_rect.width, self.exit_rect.height))

        self.cutscene = False

        self.dialog_switching = False
        self.dialog_dest = ""

    def update(self, dt):
        self.bgimage.update(dt)
        mousestate = pygame.mouse.get_pressed()
        mouse_pos = pygame.mouse.get_pos()

        if mousestate[0]:
            for bga in self.bgactors:
                if bga.dialog is not False and bga.collidepoint(mouse_pos) and self.active_dialog is False and not self.cutscene:
                    self.active_dialog = Dialog(self, bga.dialog)

        if self.active_dialog != False:
            self.active_dialog.update(dt)
            if self.active_dialog.finished:
                self.active_dialog = False

        if self.dialog_switching == True:
            self.dialog_switching = False
            self.active_dialog = False
            self.active_dialog = Dialog(self, self.dialog_dest)
            self.dialog_dest = ""

        if self.active_dialog:
            self.active_dialog.pixelated = self.pixelated

        self.bgimage.pixelated = self.pixelated

        for bga in self.bgactors:
            bga.pixelated = self.pixelated
            if not self.cutscene:
                bga.update(dt)

        if self.exit_rect.collidepoint(mouse_pos):
            t = self.text.render(self.exit_text, False, [255,155,255], [0,0,0])
            if mousestate[0] and not self.exit_click_handled:
                self.game.moving_to = 'overworld'
                self.game.is_fading = True
                self.game.fade_direction = 'out'
            self.exit_click_handled = mousestate[0]
        else:
            t = self.text.render(self.exit_text, False, [255,255,255], [0,0,0])
        size = self.text.size(self.exit_text)
        self.exit_buf.fill([0,0,0])
        self.exit_buf.blit(t, (10,10))

    def draw(self, screen):
        #screen.blit(self.bgimage, self.offset.tuple())
        self.bgimage.draw(screen)
        #self.bgactor.draw(screen)
        for bga in self.bgactors:
            bga.draw(screen)

        if len(self.blockers) > 0 and self.cutscene:
            for b in self.blockers:
                screen.blit(b['image'], b['rect'])

        if self.active_dialog != False:
            self.active_dialog.draw(screen)

        if not self.active_dialog and not self.cutscene:
            screen.blit(self.exit_buf, self.exit_rect)

    def set_dialog(self, bgactor, new_dialog):
        for b in self.bgactors:
            if b.name == bgactor:
                b.dialog = new_dialog
        self.game.set_dialog(self.name, bgactor, new_dialog)

    def moveactor(self, bgactor, dest):
        self.game.move_bgactor(self.name, bgactor, dest)
        self.bgactors = []
