# coding=utf-8
from pygame import Rect
from idle.data import * 
from idle.constants import * 

overworld_config = {
    "static": [
        {
            "name": False,
            "image": "Idle-Map-Base.png",
            "textpos": False,
            "rect": False,
        },
        {
            "name": u"Blasé Beach",
            "image": "Idle-Map-BlaseBeach.png",
            "textpos": (360, 513),
            "rect": (179, 426, 598, 600),
            "locale": "beach"
        },
        {
            "name": "Maybe I Woods",
            "image": "Idle-Map-MaybeIWoods.png",
            "textpos": (172, 103),
            "rect": (190, 75, 366, 310),
            'locale': "woods"
        },
        {
            "name": "Mel N. Collie Schoolhouse",
            "image": "Idle-Map-MelNCholieSchoolhouse.png",
            "textpos": (587,312),
            "rect": (559, 216, 789, 339),
            "locale": "school"
        },
        {
            "name": "Whatsthe Point",
            "image": "Idle-Map-WhatsthePoint.png",
            "textpos": (921, 319),
            "rect": (854, 235, 1045, 371),
            "locale": "point"
        },
        {
            "name": "Anxiety Farms",
            "image": "Idle-Map-AnxietyFarms.png",
            "textpos": (379, 223),
            "rect": (385, 137, 540, 264),
            "locale": "farm"
        }
    ],
    "animated": [
        {
            "name": "Bullshit Volcano",
            "image": "Idle-Map-BullshitVolcano-Frame%d.png",
            "offset": 2,
            "textpos": (723, 78),
            "rect": (511, 37, 915, 204),
            "locale": "volcano"
        },
        {
            "name": "Dubiety Bay",
            "image": "Idle-Map-DubietyBay-Frame%d.png",
            "offset": 1,
            "textpos": (578, 492),
            "rect": (575, 393, 945, 624),
            "locale": "bay"
        },
        {
            "name": "The Settlement",
            "image": "Idle-Map-Settlement-Frame%d.png",
            "offset": 0,
            "textpos": (415, 399),
            "rect": (373, 282, 582, 418),
            "locale": "settlement"
        },
        {
            "name": False,
            "image": "Idle-Map-Water-Frame%d.png",
            "offset":3,
            "textpos": False,
            "rect": False
        }
    ]
}

class Area(object):
    def __init__(self, text, textpos, rect, locale):
        self.text = text
        self.textpos = textpos
        if rect:
            self.rect = pygame.Rect(rect[0], rect[1], rect[2]-rect[0], rect[3]-rect[1])
        else:
            self.rect = rect
        self.locale = locale


class Overworld(object):
    def __init__(self, game):
        self.anim_count = 0
        self.areas = []
        self.rect = Rect(0, 0, RESOLUTION_X, RESOLUTION_Y)
        surf = pygame.Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()

        self.game = game
        self.pixelated = game.pixelated
        self.static = surf.copy()

        for s in overworld_config['static']:
            self.add_area(s)
            img = loadimage('overworld/' + s["image"]).convert_alpha()
            self.static.blit(img, self.rect)


        pygame.mixer.music.load(filepath('music/main_theme.wav'))
        pygame.mixer.music.play(-1)

       

        self.anim_frames = [surf.copy() for _ in range(0,4)]
        for f in self.anim_frames:
            f.fill([0,0,0,0])

        for a in overworld_config['animated']:
            img_path = [('overworld/' + a["image"]) % x for x in range(1,4)]
            self.add_area(a)
            for i, img_path in enumerate(img_path):
                image = loadimage(img_path).convert_alpha()
                
                self.anim_frames[i].blit(image, self.rect)

        self.font_size = 16
        self.text = pygame.font.Font(filepath(DIALOG_FONT), self.font_size)
        self.text_buf = surf.copy()
        self.text_buf.fill([0,0,0,0])

        self.frames = []
        for i, temp in [(x,pygame.Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()) for x in range(0,3)]:
            temp.blit(self.static, self.rect)
            temp.blit(self.anim_frames[i], self.rect)
        
            if self.pixelated > 2:
                blocks = self.pixelated * 2
                for y in range(0, RESOLUTION_Y, blocks):
                    for x in range(0, RESOLUTION_X, blocks):
                        tile = pygame.Rect([x, y, blocks, blocks])
                        color = pygame.transform.average_color(temp, tile)
                        temp.fill(color, tile)
            temp.blit(self.text_buf, self.rect)
            self.frames.append(temp)

        

        self.click_handled = False

    def update(self, dt):
        self.anim_count += dt/4 
        if self.anim_count > 3:
            self.anim_count = 0

        pos = pygame.mouse.get_pos()
        mousestate = pygame.mouse.get_pressed()
            
        for area in self.areas:
            if area.rect is False and area.text is False:
                continue
            if area.rect.collidepoint(pos):
                t = self.text.render(area.text, False, [255,155,255], [0,0,0])
                if mousestate[0] and not self.click_handled:
                    if area.locale:
                        self.game.moving_to = area.locale
                        self.game.is_fading = True
                        self.game.fade_direction = 'out'
                self.click_handled = mousestate[0]
            else:
                t = self.text.render(area.text, False, [255,255,255], [0,0,0])
            size = self.text.size(area.text)
            self.text_buf.fill([0,0,0], (   
                    area.textpos[0] - 10, 
                    area.textpos[1] - 10, 
                    size[0] + 20, 
                    size[1] + 20
            ))
            self.text_buf.blit(t, area.textpos)
        


    def draw(self, screen):
        try:
            self.frames[int(self.anim_count)].blit(self.text_buf, self.rect)
            screen.blit(self.frames[int(self.anim_count)], self.rect)
        except:
            pass

    def add_area(self, a):
        locale = False
        if 'locale' in a:
            locale = a['locale']
        self.areas.append(Area(a['name'], a['textpos'], a['rect'], locale))

    def draw_texts(self, texts):
        pass

