import pygame
from idle.constants import *
from idle.data import * 

class TextWriter(object):
    def __init__(self):
        self.curr_line = ""
        self.curr_rows = [""]
        self.curr_char = 1
        self.curr_charcount = 0
        self.curr_color = [255,255,255]

        self.font_size = 24
        self.text = pygame.font.Font(filepath(DIALOG_FONT), self.font_size)

        self.text_colors = {
            'red': [255, 0, 0],
            'white': [255, 255, 255]
        }

        self.text_width = 32 # characters that can fit on a row

        self.since_last_char = 0

        self.pixelated = 0

        self.writing_text = True
        self.scroll_wait = False
        self.aliasing = True
        self.name = False

    def update(self, dt):
        if self.curr_char < self.curr_charcount and self.since_last_char > 1:
            self.curr_char += 1
            self.since_last_char = 0
        if self.curr_char >= len(self.curr_line):
            self.writing_text = False
            self.scroll_wait = True
        self.since_last_char += dt * 20
        if self.pixelated > 0:
            self.aliasing = False


    def draw(self, screen):
        if self.curr_rows == None:
            return
        texbuf = pygame.Surface((1000, (len(self.curr_rows) + 5) * (self.font_size + 2))).convert_alpha()
        texbuf.fill([0,0,0,0])

        chars = 0
        for i, row in enumerate(self.curr_rows):
            chars += len(row)
            if self.curr_char >= chars:
                t = self.text.render(row, self.aliasing, self.curr_color)
                texbuf.blit(t, (0, (self.font_size + 2) * i + (self.font_size + 2)))
            if self.curr_char < chars:
                t = self.text.render(row[:self.curr_char-chars], self.aliasing, self.curr_color)
                texbuf.blit(t, (0, (self.font_size + 2) * i + (self.font_size + 2)))

        if self.name:
            t = self.text.render(self.name, self.aliasing, [255,255,255])
            ntile = pygame.Rect(450-10, 0, self.text.size(self.name)[0] + 20, self.font_size + 20)
            texbuf.fill([0,0,0,255], ntile)
            texbuf.blit(t, (450, 10))

        blocks = self.pixelated - 3
        if blocks > 2:
            size = texbuf.get_size()
            for y in range(0, size[1], blocks):
                for x in range(0, size[0], blocks):
                    rtile = pygame.Rect([x, y, blocks, blocks])
                    color = pygame.transform.average_color(texbuf, rtile)
                    texbuf.fill(color, rtile)
        screen.blit(texbuf, (40,20))


    def set_color(self, color):
        self.curr_color = self.text_colors[color]